/**
 * This file is part of FXGameEngine 
 * A Game Engine written in JavaFX
 * Copyright (C) 2012 Anton Epple <info@eppleton.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://opensource.org/licenses/GPL-2.0.
 * 
 * For alternative licensing or use in closed source projects contact Anton Epple 
 * <info@eppleton.de>
 */
package de.eppleton.fx2d.tilemapeditor;

import de.eppleton.fx2d.Level;
import de.eppleton.fx2d.graphicsenvironment.JavaFXGraphicsEnvironment;
import de.eppleton.fx2d.tileengine.TileMap;
import de.eppleton.fx2d.tileengine.TileMapLayer;
import de.eppleton.fx2d.tileengine.TileMapSerializationEnvironmentJAXB;
import de.eppleton.fx2d.tileengine.TileSet;
//import de.eppleton.fx2d.tileengine.algorithms.AStar;
//import de.eppleton.fx2d.tileengine.algorithms.AStar.PathNode;
import java.io.FileNotFoundException;
import java.util.List;
import java.util.logging.Logger;
import javafx.application.Application;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Orientation;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.control.Button;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.Separator;
import javafx.scene.control.SplitPane;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.control.ToolBar;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.BorderPane;
import javafx.stage.Stage;
import javax.xml.bind.JAXBException;
import net.java.html.canvas.GraphicsContext;

/**
 *
 * @author antonepple
 */
public class TileMapEditor extends Application implements ListChangeListener<TileMapLayer> {

    private TileMap tileMap;
    public LayerList layerList;
    public TileSetChooser tileSets;
    private DrawTileHandler drawTileHandler;
    private EraseHandler eraseHandler;
    public Level level;
    private Canvas canvas;
//    AStar.PathNode path;

    public String fileURL = "/Users/antonepple/NetBeansProjects/FXGames/samplegames/src/main/resources/de/eppleton/tileengine/sample/resources/maps/sample.tmx";
//    public String fileURL = "/Users/antonepple/NetBeansProjects/FXGames/tileengine/src/main/resources/assets/maps/test.tmx";
//    public String fileURL = "/assets/maps/test.tmx";
    private ToolBar initToolBar() {
        Button newMap = new Button("New");
        Button open = new Button("Open");
        Button save = new Button("Save");
        save.setOnAction(new EventHandler<ActionEvent>(){

            @Override
            public void handle(ActionEvent t) {
                try {
                    TileMapSerializationEnvironmentJAXB.writeMap(tileMap, fileURL);
                } catch (JAXBException ex) {
                    Logger.getLogger(TileMapEditor.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                } catch (FileNotFoundException ex) {
                    Logger.getLogger(TileMapEditor.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                }
            }
        });
        Separator separator = new Separator();
        ToggleButton draw = new ToggleButton("draw");
        drawTileHandler = new DrawTileHandler();
        eraseHandler = new EraseHandler();
        draw.selectedProperty().addListener(new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> ov, Boolean t, Boolean t1) {

                canvas.setOnMousePressed(drawTileHandler);
                canvas.setOnMouseDragged(drawTileHandler);
            }
        });
        ToggleButton erase = new ToggleButton("erase");
        erase.selectedProperty().addListener(new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> ov, Boolean t, Boolean t1) {
                canvas.setOnMousePressed(eraseHandler);
                canvas.setOnMouseDragged(eraseHandler);
            }
        });
        ToggleGroup group = new ToggleGroup();
        draw.setToggleGroup(group);
        erase.setToggleGroup(group);
        draw.setSelected(true);
        ToolBar toolBar = new ToolBar(newMap, open, save, separator, draw, erase);
        return toolBar;
    }

    public static void main(String[] args) {
        launch(args);
    }

    private Node initSplitPane() {
        SplitPane pane = new SplitPane();
        pane.setOrientation(Orientation.VERTICAL);
        layerList = new LayerList(tileMap);
        List<TileSet> tileSetlist = tileMap.getTileSetlist();
        tileSets = new TileSetChooser(tileSetlist);
        pane.getItems().addAll(layerList, tileSets);
        return pane;
    }

    @Override
    public void start(Stage primaryStage) throws Exception {
        try {
            TileMapSerializationEnvironmentJAXB serializer = new TileMapSerializationEnvironmentJAXB();
            tileMap = serializer.readMapFromFile(fileURL);
            canvas = new Canvas(tileMap.getTilewidth() * tileMap.getWidth(), tileMap.getHeight() * tileMap.getTileheight());
            level = new Level(new GraphicsContext(new JavaFXGraphicsEnvironment(canvas)),tileMap.getTilewidth() * tileMap.getWidth(), tileMap.getHeight() * tileMap.getTileheight(), tileMap.getTilewidth() * tileMap.getWidth(), tileMap.getHeight() * tileMap.getTileheight());
            Node rightSide = initSplitPane();
            final ObservableList<TileMapLayer> layers = layerList.getLayerList();

            for (TileMapLayer tileMapLayer : layers) {
                level.addLayer(tileMapLayer);
            }
//            level.addLayer(new AStarLayer());

            layers.addListener(this);
            BorderPane bp = new BorderPane();
            ScrollPane scrollPane = new ScrollPane();
            scrollPane.setContent(canvas);
            level.setGraphicsContext(new GraphicsContext(new JavaFXGraphicsEnvironment(canvas)));
            SplitPane split = new SplitPane();
            bp.setCenter(split);
            bp.setTop(initToolBar());
            split.getItems().addAll(scrollPane, rightSide);
            level.start();
            Scene scene = new Scene(bp, 1024, 762);
            primaryStage.setTitle("Tile Editor");
            primaryStage.setScene(scene);
            canvas.requestFocus();

            primaryStage.show();
        } catch (JAXBException ex) {
            Logger.getLogger(TileMapEditor.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }

    }

    @Override
    public void onChanged(Change<? extends TileMapLayer> change) {
        change.next();
        if (change.wasPermutated()) {
            int from = change.getFrom();
            int to = change.getTo();
        }
        if (change.wasRemoved()) {
            List<? extends TileMapLayer> removed = change.getRemoved();
            for (TileMapLayer tileMapLayer : removed) {
                level.removeLayer(tileMapLayer);
            }
        }
        if (change.wasAdded()) {
            List<? extends TileMapLayer> addedSubList = change.getAddedSubList();
            for (TileMapLayer tileMapLayer : addedSubList) {
                ObservableList<? extends TileMapLayer> list = change.getList();
                int idx = 0;
                for (TileMapLayer tileMapLayer1 : list) {
                    if (tileMapLayer == tileMapLayer1) {
                        level.addLayer(idx, tileMapLayer);
                        break;
                    }
                    idx++;
                }

            }
        }
    }

    private class EraseHandler implements EventHandler<MouseEvent> {

//        AStar.AStarTile start = new AStar.AStarTile(0, tileMap.getHeight() / 2);
//        AStar.AStarTile end = new AStar.AStarTile(tileMap.getWidth() - 1, tileMap.getHeight() / 2);

        @Override
        public void handle(MouseEvent t) {
            TileMapLayer selectedLayer = layerList.getSelected();
            if (selectedLayer == null) {
                return;
            }
            double x = t.getX();
            double y = t.getY();

            int idx = (int) ((int) x / tileMap.getTilewidth() + (((int) y / tileMap.getTileheight()) * tileMap.getWidth()));
            selectedLayer.getData().setGid(idx, 0);
//            path = AStar.getPath(tileMap, selectedLayer, start, end);
        }
    }

    private class DrawTileHandler implements EventHandler<MouseEvent> {

//        AStar.AStarTile start = new AStar.AStarTile(0, tileMap.getHeight() / 2);
//        AStar.AStarTile end = new AStar.AStarTile(tileMap.getWidth() - 1, tileMap.getHeight() / 2);

        @Override
        public void handle(MouseEvent t) {
            TileMapLayer selectedLayer = layerList.getSelected();
            if (selectedLayer == null) {
                return;
            }
            double x = t.getX();
            double y = t.getY();

            int idx = (int) ((int) x / tileMap.getTilewidth() + (((int) y / tileMap.getTileheight()) * tileMap.getWidth()));
            selectedLayer.getData().setGid(idx, tileSets.getSelectedGid());
//            path = AStar.getPath(tileMap, selectedLayer, start, end);

        }
    }

//    private class AStarLayer extends Layer {
//
//        public AStarLayer() {
//        }
//        Color pathColor ;
//
//        @Override
//        public void draw(GraphicsContext graphicsContext, double x, double y, double width, double height) {
//            PathNode start = path;
//            if (start != null) {
//                if (pathColor == null) pathColor= graphicsContext.getWebColor("rgba(255,100,100,0.2)");
//                graphicsContext.setFillStyle(pathColor);
//                graphicsContext.fillRect(start.getX() * tileMap.getTilewidth(), start.getY() * tileMap.getTileheight(), tileMap.getTilewidth(), tileMap.getTileheight());
//                while (start.getParent() != null) {
//                    start = start.getParent();
//                    graphicsContext.fillRect(start.getX() * tileMap.getTilewidth(), start.getY() * tileMap.getTileheight(), tileMap.getTilewidth(), tileMap.getTileheight());
//                }
//            } 
//        }
//    }
}
